@@ -1,26 +1,8 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Adblock Plus.
- *
- * The Initial Developer of the Original Code is
- * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * ***** END LICENSE BLOCK ***** */
+/*
+ * This Source Code is subject to the terms of the Mozilla Public License
+ * version 2.0 (the "License"). You can obtain a copy of the License at
+ * http://mozilla.org/MPL/2.0/.
+ */
 
 let nodes = null;
 let item = null;
@@ -150,6 +132,8 @@ function init()
   E("firstParty").hidden = thirdParty;
 
   let typeGroup = E("typeGroup");
+  let defaultTypes = RegExpFilter.prototype.contentType & ~RegExpFilter.typeMap.DOCUMENT;
+  let isDefaultType = (RegExpFilter.typeMap[item.typeDescr] & defaultTypes) != 0;
   for each (let type in types)
   {
     if (type == Policy.type.ELEMHIDE)
@@ -158,10 +142,15 @@ function init()
     let typeNode = document.createElement("checkbox");
     typeNode.setAttribute("value", Policy.typeDescr[type].toLowerCase().replace(/\_/g, "-"));
     typeNode.setAttribute("label", Policy.localizedDescr[type].toLowerCase());
-    typeNode.setAttribute("checked", "true");
+
+    let typeMask = RegExpFilter.typeMap[Policy.typeDescr[type]];
+    typeNode._defaultType = (typeMask & defaultTypes) != 0;
+    if ((isDefaultType && typeNode._defaultType) || (!isDefaultType && item.type == type))
+      typeNode.setAttribute("checked", "true");
+
     if (item.type == type)
       typeNode.setAttribute("disabled", "true");
-    typeNode.addEventListener("command", updateFilter, false);
+    typeNode.addEventListener("command", function() checkboxUpdated(this), false);
     typeGroup.appendChild(typeNode);
   }
 
@@ -177,6 +166,12 @@ function init()
   updatePatternSelection();
 }
 
+function checkboxUpdated(checkbox)
+{
+  checkbox._lastChange = Date.now();
+  updateFilter();
+}
+
 function updateFilter()
 {
   let filter = "";
@@ -205,45 +200,59 @@ function updateFilter()
     {
       let domainRestriction = E("domainRestriction").value.replace(/[,\s]/g, "").replace(/\.+$/, "");
       if (domainRestriction)
-        options.push("domain=" + domainRestriction);
+        options.push([E("domainRestrictionEnabled")._lastChange || 0, "domain=" + domainRestriction]);
     }
 
     if (E("firstParty").checked)
-      options.push("~third-party");
+      options.push([E("firstParty")._lastChange || 0, "~third-party"]);
     if (E("thirdParty").checked)
-      options.push("third-party");
+      options.push([E("thirdParty")._lastChange || 0, "third-party"]);
 
     if (E("matchCase").checked)
-      options.push("match-case");
+      options.push([E("matchCase")._lastChange || 0, "match-case"]);
 
     let collapse = E("collapse");
     disableElement(collapse, type == "whitelist", "value", "");
     if (collapse.value != "")
-      options.push(collapse.value);
+      options.push([collapse._lastChange, collapse.value]);
 
     let enabledTypes = [];
     let disabledTypes = [];
+    let forceEnabledTypes = [];
     for (let typeNode = E("typeGroup").firstChild; typeNode; typeNode = typeNode.nextSibling)
     {
       let value = typeNode.getAttribute("value");
       if (value == "document")
         disableElement(typeNode, type != "whitelist", "checked", false);
 
-      if (value != "document" || !typeNode.disabled)
+      if (!typeNode._defaultType)
       {
-        if (typeNode.checked)
-          enabledTypes.push(value);
-        else
-          disabledTypes.push("~" + value);
+        if (typeNode.getAttribute("checked") == "true")
+          forceEnabledTypes.push([typeNode._lastChange || 0, value]);
       }
+      else if (typeNode.getAttribute("checked") == "true")
+        enabledTypes.push([typeNode._lastChange || 0, value]);
+      else
+        disabledTypes.push([typeNode._lastChange || 0, "~" + value]);
     }
-    if (disabledTypes.length < enabledTypes.length)
+    if (!forceEnabledTypes.length && disabledTypes.length < enabledTypes.length)
       options.push.apply(options, disabledTypes);
     else
       options.push.apply(options, enabledTypes);
+    options.push.apply(options, forceEnabledTypes);
 
     if (options.length)
-      filter += "$" + options.join(",");
+    {
+      options.sort(function(a, b) a[0] - b[0]);
+      filter += "$" + options.map(function(o) o[1]).join(",");
+    }
+  }
+  else
+  {
+    let defaultTypes = RegExpFilter.prototype.contentType & ~RegExpFilter.typeMap.DOCUMENT;
+    let isDefaultType = (RegExpFilter.typeMap[item.typeDescr] & defaultTypes) != 0;
+    if (!isDefaultType)
+      filter += "$" + item.typeDescr.toLowerCase().replace(/\_/g, "-");
   }
 
   filter = Filter.normalize(filter);
@@ -264,11 +273,7 @@ function updateFilter()
 
   if (E("disabledWarning").hidden)
   {
-    let subscription = null;
-    for each (let s in FilterStorage.subscriptions)
-      if (s instanceof SpecialSubscription && s.isFilterAllowed(compiledFilter) && (!subscription || s.priority > subscription.priority))
-        subscription = s;
-
+    let subscription = FilterStorage.getGroupForFilter(compiledFilter);
     let warning = E("groupDisabledWarning");
     if (subscription && subscription.disabled)
     {
@@ -322,7 +327,7 @@ function updatePatternSelection()
 
   function testFilter(/**String*/ filter) /**Boolean*/
   {
-    return RegExpFilter.fromText(filter).matches(item.location, item.typeDescr, item.docDomain, item.thirdParty);
+    return RegExpFilter.fromText(filter + "$" + item.typeDescr).matches(item.location, item.typeDescr, item.docDomain, item.thirdParty);
   }
 
   let anchorStartCheckbox = E("anchorStart");
@@ -354,15 +359,9 @@ function updateCustomPattern()
 
 function addFilter() {
   let filter = Filter.fromText(document.getElementById("filter").value);
-
-  if (filter.disabled)
-  {
-    filter.disabled = false;
-    FilterStorage.triggerObservers("filters enable", [filter]);
-  }
+  filter.disabled = false;
 
   FilterStorage.addFilter(filter);
-  FilterStorage.saveToDisk();
 
   if (nodes)
     Policy.refilterNodes(nodes, item);
@@ -383,17 +382,18 @@ function setAdvancedMode(mode) {
 }
 
 function disableElement(element, disable, valueProperty, disabledValue) {
-  if (element.disabled == disable)
+  if ((element.getAttribute("disabled") == "true") == disable)
     return;
 
-  element.disabled = disable;
   if (disable)
   {
+    element.setAttribute("disabled", "true");
     element._abpStoredValue = element[valueProperty];
     element[valueProperty] = disabledValue;
   }
   else
   {
+    element.removeAttribute("disabled");
     if ("_abpStoredValue" in element)
       element[valueProperty] = element._abpStoredValue;
     delete element._abpStoredValue;
@@ -401,7 +401,7 @@ function disableElement(element, disable, valueProperty, disabledValue) {
 }
 
 function openPreferences() {
-  Utils.openSettingsDialog(item.location, E("filter").value);
+  Utils.openFiltersDialog(Filter.fromText(E("filter").value));
 }
 
 function doEnable() {
@@ -412,8 +412,6 @@ function doEnable() {
 function enableSubscription(subscription)
 {
   subscription.disabled = false;
-  FilterStorage.triggerObservers("subscriptions enable", [subscription]);
-  FilterStorage.saveToDisk();
   E("groupDisabledWarning").hidden = true;
 }
 
