@@ -1,26 +1,8 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Adblock Plus.
- *
- * The Initial Developer of the Original Code is
- * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * ***** END LICENSE BLOCK ***** */
+/*
+ * This Source Code is subject to the terms of the Mozilla Public License
+ * version 2.0 (the "License"). You can obtain a copy of the License at
+ * http://mozilla.org/MPL/2.0/.
+ */
 
 //
 // Report data template, more data will be added during data collection
@@ -36,6 +18,7 @@ let reportData =
       <option id="objecttabs">{Prefs.frameobjects}</option>
       <option id="collapse">{!Prefs.fastcollapse}</option>
       <option id="privateBrowsing">{Prefs.privateBrowsing}</option>
+      <option id="subscriptionsAutoUpdate">{Prefs.subscriptions_autoupdate}</option>
     </options>
     <window/>
     <requests/>
@@ -50,7 +33,6 @@ let reportData =
 
 let reportsListDataSource =
 {
-  json: Cc["@mozilla.org/dom/json;1"].createInstance(Ci.nsIJSON),
   list: [],
 
   collectData: function(wnd, windowURI, callback)
@@ -58,7 +40,7 @@ let reportsListDataSource =
     let data = null;
     try
     {
-      data = this.json.decode(Prefs.recentReports);
+      data = JSON.parse(Prefs.recentReports);
     }
     catch (e)
     {
@@ -120,7 +102,7 @@ let reportsListDataSource =
     this.list.unshift({site: site, reportURL: reportURL, time: Date.now()});
     try
     {
-      Prefs.recentReports = this.json.encode(this.list);
+      Prefs.recentReports = JSON.stringify(this.list);
     }
     catch (e)
     {
@@ -131,7 +113,7 @@ let reportsListDataSource =
   clear: function()
   {
     this.list = [];
-    Prefs.recentReports = this.json.encode(this.list);
+    Prefs.recentReports = JSON.stringify(this.list);
     E("recentReports").hidden = true;
   },
 
@@ -224,7 +206,7 @@ let filtersDataSource =
       {
         let filter = Filter.fromText(f)
         let hitCount = wndStats.filters[f];
-        filters.appendChild(<filter text={filter.text} subscriptions={filter.subscriptions.filter(function(s) !s.disabled).map(function(s) s.url).join(" ")} hitCount={hitCount}/>);
+        filters.appendChild(<filter text={filter.text} subscriptions={filter.subscriptions.filter(subscriptionsDataSource.subscriptionFilter).map(function(s) s.url).join(" ")} hitCount={hitCount}/>);
         this.origFilters.push(filter);
       }
     }
@@ -234,6 +216,15 @@ let filtersDataSource =
 
 let subscriptionsDataSource =
 {
+  subscriptionFilter: function(s)
+  {
+    if (s.disabled || !(s instanceof RegularSubscription))
+      return false;
+    if (s instanceof DownloadableSubscription && !/^(http|https|ftp):/i.test(s.url))
+      return false;
+    return true;
+  },
+
   collectData: function(wnd, windowURI, callback)
   {
     let subscriptions = reportData.subscriptions;
@@ -241,7 +232,7 @@ let subscriptionsDataSource =
     for (let i = 0; i < FilterStorage.subscriptions.length; i++)
     {
       let subscription = FilterStorage.subscriptions[i];
-      if (subscription.disabled || !(subscription instanceof RegularSubscription))
+      if (!this.subscriptionFilter(subscription))
         continue;
 
       let subscriptionXML = <subscription id={subscription.url} disabledFilters={subscription.filters.filter(function(filter) filter instanceof ActiveFilter && filter.disabled).length}/>;
@@ -255,7 +246,6 @@ let subscriptionsDataSource =
           subscriptionXML.@softExpiration = subscription.softExpiration - now;
         if (subscription.expires)
           subscriptionXML.@hardExpiration = subscription.expires - now;
-        subscriptionXML.@autoDownloadEnabled = subscription.autoDownload;
         subscriptionXML.@downloadStatus = subscription.downloadStatus;
       }
       subscriptions.appendChild(subscriptionXML);
@@ -696,9 +686,17 @@ let issuesDataSource =
   disabledFilters: [],
   disabledSubscriptions: [],
   ownFilters: [],
-  numSubscriptions: FilterStorage.subscriptions.filter(function(subscription) subscription instanceof DownloadableSubscription && !subscription.disabled).length,
+  numSubscriptions: 0,
   numAppliedFilters: Infinity,
 
+  subscriptionFilter: function(s)
+  {
+    if (s instanceof DownloadableSubscription)
+      return subscriptionsDataSource.subscriptionFilter(s);
+    else
+      return false;
+  },
+
   collectData: function(wnd, windowURI, callback)
   {
     this.contentWnd = wnd;
@@ -759,6 +757,7 @@ let issuesDataSource =
         }
       }
 
+      this.numSubscriptions = FilterStorage.subscriptions.filter(this.subscriptionFilter).length;
       this.numAppliedFilters = 0;
       for each (let filter in filtersDataSource.origFilters)
       {
@@ -885,11 +884,8 @@ let issuesDataSource =
 
   removeWhitelist: function()
   {
-    if (this.whitelistFilter && this.whitelistFilter.subscriptions.length && !this.whitelistFilter.disabled)
-    {
+    if (this.whitelistFilter && this.whitelistFilter.subscriptions.length)
       this.whitelistFilter.disabled = true;
-      FilterStorage.triggerObservers("filters disable", [this.whitelistFilter]);
-    }
     E("issuesWhitelistBox").hidden = true;
     this.forceReload();
   },
@@ -920,21 +916,12 @@ let issuesDataSource =
     
       FilterStorage.addSubscription(subscription);
 
-      if (subscription.disabled)
-      {
-        subscription.disabled = false;
-        FilterStorage.triggerObservers("subscriptions enable", [subscription]);
-      }
-
+      subscription.disabled = false;
       subscription.title = title;
-      if (subscription instanceof DownloadableSubscription)
-        subscription.autoDownload = result.autoDownload;
-      FilterStorage.triggerObservers("subscriptions updateinfo", [subscription]);
     
       if (subscription instanceof DownloadableSubscription && !subscription.lastDownload)
         Synchronizer.execute(subscription);
     }
-    FilterStorage.saveToDisk();
 
     E("issuesNoSubscriptionsBox").hidden = true;
     this.forceReload();
@@ -943,11 +930,8 @@ let issuesDataSource =
   disableFilter: function(node)
   {
     let filter = node.abpFilter;
-    if (filter && filter.subscriptions.length && !filter.disabled)
-    {
+    if (filter && filter.subscriptions.length)
       filter.disabled = true;
-      FilterStorage.triggerObservers("filters disable", [filter]);
-    }
 
     node.parentNode.removeChild(node);
     if (!E("issuesOwnFilters").firstChild)
@@ -958,11 +942,8 @@ let issuesDataSource =
   enableFilter: function(node)
   {
     let filter = node.abpFilter;
-    if (filter && filter.subscriptions.length && filter.disabled)
-    {
+    if (filter && filter.subscriptions.length)
       filter.disabled = false;
-      FilterStorage.triggerObservers("filters enable", [filter]);
-    }
 
     node.parentNode.removeChild(node);
     if (!E("issuesDisabledFilters").firstChild)
@@ -974,11 +955,8 @@ let issuesDataSource =
   enableSubscription: function(node)
   {
     let subscription = node.abpSubscription;
-    if (subscription && subscription.disabled)
-    {
+    if (subscription)
       subscription.disabled = false;
-      FilterStorage.triggerObservers("subscriptions enable", [subscription]);
-    }
 
     node.parentNode.removeChild(node);
     if (!E("issuesDisabledSubscriptions").firstChild)
@@ -1252,7 +1230,7 @@ function reportSent(event)
 {
   let request = event.target;
   let success = false;
-  let errorMessage = Utils.getString("synchronize_connection_error");
+  let errorMessage = E("sendReportError").getAttribute("defaultError");
   try
   {
     let status = request.channel.status;
@@ -1358,8 +1336,7 @@ function processLinkClick(event)
 
 function copyLink(url)
 {
-  let clipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
-  clipboardHelper.copyString(url);
+  Utils.clipboardHelper.copyString(url);
 }
 
 function censorURL(url)
