@@ -1,26 +1,8 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Adblock Plus.
- *
- * The Initial Developer of the Original Code is
- * Wladimir Palant.
- * Portions created by the Initial Developer are Copyright (C) 2006-2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * ***** END LICENSE BLOCK ***** */
+/*
+ * This Source Code is subject to the terms of the Mozilla Public License
+ * version 2.0 (the "License"). You can obtain a copy of the License at
+ * http://mozilla.org/MPL/2.0/.
+ */
 
 Cu.import("resource://gre/modules/XPCOMUtils.jsm");
 
@@ -92,7 +74,7 @@ function init() {
 
   // Initialize matcher for disabled filters
   reloadDisabledFilters();
-  FilterStorage.addObserver(reloadDisabledFilters);
+  FilterNotifier.addListener(reloadDisabledFilters);
   Prefs.addListener(onPrefChange);
 
   // Activate flasher
@@ -114,8 +96,9 @@ function mainUnload() {
 function cleanUp() {
   flasher.stop();
   requestNotifier.shutdown();
-  FilterStorage.removeObserver(reloadDisabledFilters);
+  FilterNotifier.removeListener(reloadDisabledFilters);
   Prefs.removeListener(onPrefChange);
+  E("list").view = null;
 
   abpHooks.getBrowser().removeProgressListener(progressListener);
   mainWin.removeEventListener("unload", mainUnload, false);
@@ -131,12 +114,25 @@ function onPrefChange(name)
     reloadDisabledFilters();
 }
 
+let reloadDisabledScheduled = false;
+
 /**
  * Updates matcher for disabled filters (global disabledMatcher variable),
- * called on each filter change.
+ * called on each filter change. Execute delayed to prevent multiple subsequent
+ * invocations.
  */
 function reloadDisabledFilters()
 {
+  if (reloadDisabledScheduled)
+    return;
+
+  Utils.runAsync(reloadDisabledFiltersInternal);
+  reloadDisabledScheduled = true;
+}
+
+function reloadDisabledFiltersInternal()
+{
+  reloadDisabledScheduled = false;
   disabledMatcher.clear();
 
   if (Prefs.enabled)
@@ -270,7 +266,7 @@ function fillInTooltip(e) {
 
   var showPreview = Prefs.previewimages && !("tooltip" in item);
   showPreview = showPreview && item.typeDescr == "IMAGE";
-  showPreview = showPreview && (!item.filter || item.filter instanceof WhitelistFilter);
+  showPreview = showPreview && (!item.filter || item.filter.disabled || item.filter instanceof WhitelistFilter);
   if (showPreview) {
     // Check whether image is in cache (stolen from ImgLikeOpera)
     if (!cacheSession) {
@@ -426,7 +422,8 @@ function doBlock() {
   openDialog("chrome://adblockplus/content/ui/composer.xul", "_blank", "chrome,centerscreen,resizable,dialog=no,dependent", item.nodes, item.orig);
 }
 
-function editFilter() {
+function editFilter()
+{
   var item = treeView.getSelectedItem();
   if (treeView.data && !treeView.data.length)
     item = treeView.getDummyTooltip();
@@ -437,13 +434,11 @@ function editFilter() {
   if (!("location") in item)
     item.location = undefined
 
-  Utils.openSettingsDialog(item.location, item.filter);
+  Utils.openFiltersDialog(item.filter);
 }
 
 function enableFilter(filter, enable) {
   filter.disabled = !enable;
-  FilterStorage.triggerObservers(enable ? "filters enable" : "filters disable", [filter]);
-  FilterStorage.saveToDisk();
 
   treeView.boxObject.invalidate();
 }
@@ -500,17 +495,15 @@ function disableOnSite(item, /**Filter*/ filter, /**String*/ domain)
   // Insert new filter before the old one and remove the old one then
   let newFilter = Filter.fromText(text);
   if (newFilter.disabled && newFilter.subscriptions.length)
-  {
     newFilter.disabled = false;
-    FilterStorage.triggerObservers("filters enable", [newFilter]);
-  }
   else if (!newFilter.subscriptions.length)
   {
     newFilter.disabled = false;
-    FilterStorage.addFilter(newFilter, filter);
+    let subscription = filter.subscriptions.filter(function(s) s instanceof SpecialSubscription)[0];
+    if (subscription)
+      FilterStorage.addFilter(newFilter, subscription, subscription.filters.indexOf(filter));
   }
   FilterStorage.removeFilter(filter);
-  FilterStorage.saveToDisk();
 
   // Update display
   item.filter = null;
@@ -522,8 +515,7 @@ function copyToClipboard() {
   if (!items.length)
     return;
 
-  var clipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
-  clipboardHelper.copyString(items.map(function(item) {return item.location}).join(Utils.getLineBreak()));
+  Utils.clipboardHelper.copyString(items.map(function(item) {return item.location}).join(Utils.getLineBreak()));
 }
 
 function copyFilter() {
@@ -534,8 +526,7 @@ function copyFilter() {
   if (!items.length)
     return;
 
-  var clipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
-  clipboardHelper.copyString(items.map(function(item) {return item.filter.text}).join(Utils.getLineBreak()));
+  Utils.clipboardHelper.copyString(items.map(function(item) {return item.filter.text}).join(Utils.getLineBreak()));
 }
 
 function selectAll() {
@@ -662,9 +653,20 @@ function compareDocDomain(item1, item2)
     return 0;
 }
 
+function compareFilterSource(item1, item2)
+{
+  let subs1 = item1.filter ? item1.filter.subscriptions.map(function(s) s.title).join(", ") : "";
+  let subs2 = item2.filter ? item2.filter.subscriptions.map(function(s) s.title).join(", ") : "";
+  if (subs1 < subs2)
+    return -1;
+  else if (subs1 > subs2)
+    return 1;
+  else
+    return 0;
+}
+
 function createSortWithFallback(cmpFunc, fallbackFunc, desc) {
   var factor = (desc ? -1 : 1);
-
   return function(item1, item2) {
     var ret = cmpFunc(item1, item2);
     if (ret == 0)
@@ -709,15 +711,12 @@ var treeView = {
   setTree: function(boxObject) {
     if (!boxObject)
       return;
-
     this.boxObject = boxObject;
     this.itemsDummy = boxObject.treeBody.getAttribute("noitemslabel");
     this.whitelistDummy = boxObject.treeBody.getAttribute("whitelistedlabel");
-
-    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "col-size", "col-docDomain", "state-regular", "state-filtered", "state-whitelisted", "state-hidden"];
+    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "col-size", "col-docDomain", "col-filterSource", "state-regular", "state-filtered", "state-whitelisted", "state-hidden"];
     var boolAtoms = ["selected", "dummy", "filter-disabled"];
     var atomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
-
     this.atoms = {};
     for each (let atom in stringAtoms)
       this.atoms[atom] = atomService.getAtom(atom);
@@ -770,21 +769,16 @@ var treeView = {
     // Prevent a reference through closures
     boxObject = null;
   },
-
   get rowCount() {
     return (this.data && this.data.length ? this.data.length : 1);
   },
-
   getCellText: function(row, col) {
     col = col.id;
-
-    if (col != "type" && col != "address" && col != "filter" && col != "size" && col != "docDomain")
+    if (col != "type" && col != "address" && col != "filter" && col != "size" && col != "docDomain" && col != "filterSource")
       return "";
-
     if (this.data && this.data.length) {
       if (row >= this.data.length)
         return "";
-
       if (col == "type")
         return this.data[row].localizedDescr;
       else if (col == "filter")
@@ -796,6 +790,13 @@ var treeView = {
       }
       else if (col == "docDomain")
         return this.data[row].docDomain + " " + (this.data[row].thirdParty ? docDomainThirdParty : docDomainFirstParty);
+      else if (col == "filterSource")
+      {
+        if (!this.data[row].filter)
+          return "";
+
+        return this.data[row].filter.subscriptions.filter(function(s) !s.disabled).map(function(s) s.title).join(", ");
+      }
       else
         return this.data[row].location;
     }
@@ -803,7 +804,6 @@ var treeView = {
       // Empty list, show dummy
       if (row > 0 || (col != "address" && col != "filter"))
         return "";
-
       if (col == "filter") {
         var filter = Policy.isWindowWhitelisted(window.content);
         return filter ? filter.text : "";
@@ -951,12 +951,12 @@ var treeView = {
     size: createSortWithFallback(compareSize, sortByAddress, false),
     sizeDesc: createSortWithFallback(compareSize, sortByAddress, true),
     docDomain: createSortWithFallback(compareDocDomain, sortByAddress, false),
-    docDomainDesc: createSortWithFallback(compareDocDomain, sortByAddress, true)
+    docDomainDesc: createSortWithFallback(compareDocDomain, sortByAddress, true),
+    filterSource: createSortWithFallback(compareFilterSource, sortByAddress, false),
+    filterSourceDesc: createSortWithFallback(compareFilterSource, sortByAddress, true)
   },
-
   clearData: function(data) {
     var oldRows = this.rowCount;
-
     this.allData = [];
     this.dataMap = {__proto__: null};
     this.refilter();
@@ -1074,6 +1074,7 @@ var treeView = {
 
     return (item.location.toLowerCase().indexOf(this.filter) >= 0 ||
             (item.filter && item.filter.text.toLowerCase().indexOf(this.filter) >= 0) ||
+            item.typeDescr.toLowerCase().indexOf(this.filter) >= 0 ||
             item.localizedDescr.toLowerCase().indexOf(this.filter) >= 0 ||
             (item.docDomain && item.docDomain.toLowerCase().indexOf(this.filter) >= 0) ||
             (item.docDomain && item.thirdParty && docDomainThirdParty.toLowerCase().indexOf(this.filter) >= 0) ||
