@@ -15,7 +15,8 @@ system("hg", "clone", "https://hg.adblockplus.org/jsdoc-toolkit/") unless -e "js
          '-d=' . $target,
          '-a',
          '-p',
-         'chrome/content/', 'components/AdblockPlus.js');
+         '-x=js,jsm',
+         'modules/', 'components/Initializer.js');
 
 $0 = "jsdoc-toolkit/jsrun.pl";
 do $0;
